module Glossync
  # Holds various missing translation handlers.  Any object is a missing
  # translation handler if it responds to the method handle with an arity of 4
  module MissingTranslationHandler

    # The default handler, simply returns the error message
    class Default
      # handles an I18n MissingTranslation error
      # @param error the error thrown by I18n, should always be a MissingTranslation
      # @param who the self of the class that caught the error
      # @param field the field that has a missing translation
      # @param locale the locale that was being translated to
      # @return [String]
      def handle(error, _who, _field, _locale)
        error.message
      end
    end

    # Raises the error message
    class Raiser
      # :nodoc:
      def handle(error, _, _, _)
        raise error
      end
    end

    # This handler attempts to translate the given string to the target language using
    # the Google Translate API
    class GoogleTranslate
      # @param key the key for the Google Translate API
      def initialize(key)
        @api_key = key
        @cache = {}
      end

      # Translate text to the to locale
      # @param text the text to translate
      # @param to the locale to translate to
      # @return [String] the translated text or an error message
      def translate(text, to)
        EasyTranslate.translate(
          text,
          from: Glossync.options[:base_locale],
          to: to,
          key: @api_key
        )
      rescue SocketError
        "failed to connect to google translate api"
      end

      # :nodoc:
      def handle(_, who, field, locale)
        key = who.tl_key(field)

        @cache[key] = translate(who[field], locale) unless @cache.key?(key)

        @cache[key]
      end
    end
  end
end
